
//////////////////////////////////////////////////////////////////////
//
// File: sevensegment.c
//
// Author: J F Main.
//
// Description:
//
// Constant time seven segment display routines for time critical code.
//
// A 4017 johnson counter selects which seven segment is active using a
// transistor to supply current at the common cathode of the selected segment.
//
// The first output of the 4017 is not used so a reset control to the
// 4017 clears all output.
//
// Version: 1.00 - Initial release.
//
// Copyright : Copyright  John Main 2006
//   http://www.best-microcontroller-projects.com
//   Free for non commercial use as long as this entire copyright notice
//   is included in source code and any other documentation.
//
//////////////////////////////////////////////////////////////////////
#include "def.h"
#include "bit.h"
#include "gate.h"
#include "delay.h"

extern volatile unsigned short sdmy; // defined in top file

//////////////////////////////////////////////////////////////////////
//
// Must execute in a constant time execution every call.
//
// Using 'case' or 'if' may cause non constant time execution.
//
// This routine only accomodates character numbers '0'-'9'.
//
unsigned short charNumTo7SegMask(unsigned char chr) org 48 {
unsigned short val;

// need a lookup table for const time action
unsigned short ret[10] = { 0x3F, 0x06, 0x5B, 0x4F, 0x66,
                           0x6D, 0x7D, 0x07, 0x7F, 0x6F };
   val = ret[chr-'0'];
   
   #ifdef PATH2
      asm { CLRW }
   #endif

   if (chr>'9') {
      val = 00;
   } else {
      sdmy = 00;
      asm {
         CLRW   // constant time
      }
   }
   // keep these 2 statements separate for const time because
   // for C if the first condition is true - the next is not tested.

   #ifdef PATH2
      asm { CLRW } // landing point to let 5.0.0.3 simulator update
   #endif

   if (chr<'0') {
      val = 00;
   } else {
      sdmy = 00;
      asm {
         CLRW   // constant time
      }
   }  // keep these 2 statements separate for const time

   return val;
}

////////////////////////////////////////////////////////////////////////
// display a string on 8 x 7 segment display.
// display 1 character at a time
// to allow controlling routine to determine length of time display
// digit is held on.
// must use SEG_RESET as needed in main routine (if at all).
//
// Arguments:
//   s - the character string to display
//
// Note:
//   Hardware is a 4017 with outputs from Q1
//  (Q0 unused so reset=display off).
//  Outputs go from Q0 to Q9 - 10 output.
//
void init_str_8seg7(void) {
   SEG_RESET;       // generate reset strobe.
   NEXT_SEG_STROBE; // N.B. 1st output of 4017 is not used.
}

////////////////////////////////////////////////////////////////////////
//
void display_str_8seg7(char *s, unsigned short dpIdx) org 151 {
volatile unsigned short val; // do not optimise out.
static unsigned short idx=0; // segment index counter
unsigned short zero = 0; // force compiler to do same i.e. do not let it use CLRF

// must have 10 chars as hardware has an extra unused output
// to allow constant timing do not want to do:
//   if (idx==8) { SEG_RESET; } // recode this to free run.
// reset once then keep calling this with no 'if' test.

   idx++;

   #ifdef PATH2
      asm { CLRW } // landing point to let 5.0.0.3 simulator update
   #endif

   // idx = idx % 10; // this has to do a division so = longer & non constant time
   if (idx==10) {    // compare to literal does more work.
      idx = zero;
   } else {
      sdmy = zero;
      asm {
        CLRW
      }
   }

   val = charNumTo7SegMask( s[idx] );

   #ifdef PATH2
      asm { CLRW }
   #endif

   if (idx==dpIdx) {
      val |= 0x80; // decimal place
   } else {
      sdmy |= 0x80;  // volatile so this is kept = const time.
      asm {
        CLRW
      }
   }

   PORTD=val;

   NEXT_SEG_STROBE; //NB 1st output is not used (of 4017).
}





